#ifndef CREFL171_COMMON_H
#define CREFL171_COMMON_H 1

#include <stdio.h>
#include <mfhdf.h>
#include <stdlib.h>
#include <string.h>
#include <hdf5.h>

#define PROCESS_VERSION_NUMBER "1.7.1"

#define MAXNAMELENGTH 200

#define Nbands 16
#define REFLMIN -0.01
#define REFLMAX 1.6
#define MAXSOLZ 86.5
#define DEG2RAD	0.0174532925199	/* PI/180 */

/* =============================================================================== */
/* Enum declarations                                                               */
/* =============================================================================== */

enum {MODIS_1KM   = 0, VIIRS_I1 = 0, 
      MODIS_HKM   = 1, VIIRS_I2 = 1, 
      MODIS_QKM   = 2, VIIRS_M7 = 2, 
      VIIRS_M3    = 3, 
      VIIRS_M4    = 4, 
      VIIRS_M8    = 5, 
      VIIRS_I3    = 6, 
      VIIRS_M10   = 7, 
      VIIRS_M11   = 8, 
      VIIRS_M5    = 9,
      VIIRS_M2    = 10,
      VIIRS_GMTCO = 11, 
      VIIRS_GMODO = 12, 
      VIIRS_GITCO = 13, 
      NR_FILES,
      INPUT_UNKNOWN};

enum {BAND1  = 0,  I1  = 0, 
      BAND2  = 1,  I2  = 1,
      BAND3  = 2,  M7  = 2, 
      BAND4  = 3,  M3  = 3, 
      BAND5  = 4,  M4  = 4, 
      BAND6  = 5,  M8  = 5, 
      BAND7  = 6,  I3  = 6, 
      BAND8  = 7,  M10 = 7, 
      BAND9  = 8,  M11 = 8, 
      BAND10 = 9,  M5  = 9, 
      BAND11 = 10, M2 = 10,
      BAND12, BAND13, BAND14, BAND15, BAND16, SOLZ, SENZ, SOLA, SENA, LON, LAT, Nitems};

/* =============================================================================== */
/* Structure declarations                                                          */
/* =============================================================================== */

typedef struct
{
	char *name, *filename;
	int32 file_id, id, index, num_type, rank, n_attr, Nl, Np, rowsperscan;
	int32 start [H4_MAX_VAR_DIMS], edges [H4_MAX_VAR_DIMS], dim_sizes [H4_MAX_VAR_DIMS];
	void *data, *fillvalue;
	float64 factor, offset;
    /* CREFL has coefficients for the first 16 MODIS bands. Each dataset is associated 
       with a specific MODIS band number. The crefl_band_number variable is used to 
       indicate what CREFL band number is associated with the dataset. */
    char crefl_band_number, sensor_band_number; 
    
    /* Variables for HDF5 data */
    hid_t h5_file_id, h5_dataset_id, h5_dataspace_id, h5_memspace_id, h5_datatype;
    hsize_t h5_start [2], h5_stride [2], h5_count [2];
} SDS;

typedef struct
{
   int output1km, output500m, append, bands, gzip, nearest, overwrite, sealevel, TOA, verbose;
   float reflmin, reflmax, maxsolz;
   char *input_files [NR_FILES];
   int32 input_file_id_arr [NR_FILES];
   unsigned char process[Nbands];
} crefl_in_options_st;

void init_crefl_in_options_st (crefl_in_options_st *options_val_st);

/*
Parse band list and set relevant elements in process[] array.
Returns non-zero if invalid band(s) specified, 0 otherwise (i.e., success).
*/
int modis_parse_bands (char *bandstr, unsigned char process[Nbands]);
/*
Returns 0 if xmin <= x <= xmax, non-zero otherwise.
*/
int range_check (float x, float xmin, float xmax);

void usage (void);

void dem_init_data (crefl_in_options_st *options_val_st, SDS *dem_sds_st);
void dem_finalize_data (crefl_in_options_st *options_val_st, SDS *dem_sds_st);

#endif /* CREFL171_COMMON_H */

